#!/bin/sh

set -e -o pipefail

# Init Java options - ORG_JAVA_OPTS will keep the original value when the script was started for the first time
JAVA_OPTS="${ORG_JAVA_OPTS:=${JAVA_OPTS:- }}"
export ORG_JAVA_OPTS

# Allow custom props to be inserted
#test -f /custom-conf1.sh && . /custom-conf1.sh

# General Java options
JAVA_OPTS+=" -showversion"
JAVA_OPTS+=" -XX:+PrintFlagsFinal"
JAVA_OPTS+=" -XX:+PrintCommandLineFlags"
JAVA_OPTS+=" -XX:+PrintStringTableStatistics"
JAVA_OPTS+=" -XX:ErrorFile=/tmp/error_java_identity_%p.log"
JAVA_OPTS+=" -Xlog:gc*:/tmp/identity.txt:time,level,tags:filesize=50M,filecount=1"

# memory options
# use parallel GC if no GC is added in JAVA_OPTS - the default G1 uses too much off-heap memory
if ! grep -E 'UseG1GC|UseConcMarkSweepGC|UseEpsilonGC|UseParallelGC|UseSerialGC|UseZGC' <<< "$JAVA_OPTS" > /dev/null
then
  echo "Setting GC to ParallelGC"
  JAVA_OPTS+=" -XX:+UseParallelGC"
fi

JAVA_OPTS+=" -Xms${JVM_XMS:-768m} -Xmx${JVM_XMX:-1024m}"

echo "Setting metaspace to ${JVM_METASPACE:=256m}"
JAVA_OPTS+=" -XX:MaxMetaspaceSize=$JVM_METASPACE"

JAVA_OPTS+=" -Xss384k"
JAVA_OPTS+=" -XX:MaxJavaStackTraceDepth=0"
JAVA_OPTS+=" -XX:SurvivorRatio=8"
JAVA_OPTS+=" -XX:TargetSurvivorRatio=90"
JAVA_OPTS+=" -XX:MaxTenuringThreshold=15"
JAVA_OPTS+=" -XX:ParallelGCThreads=4"
JAVA_OPTS+=" -XX:+OptimizeStringConcat"

# https://jira.eng.vmware.com/browse/VEBS-340
HEAP_DUMP_PATH="/heapDump/$( date '+%Y-%m-%d_%H-%M-%S' )_heap_dump.hprof"
JAVA_OPTS+=" -XX:+HeapDumpOnOutOfMemoryError -XX:HeapDumpPath=$HEAP_DUMP_PATH"
JAVA_OPTS+=" -XX:+ExitOnOutOfMemoryError"


# Configure Datasource
DATASOURCE_URI=""
if [ -n "$DATASOURCE_URL" ]; then
  DATASOURCE_URI=$DATASOURCE_URL
fi

if [ -z "$DATASOURCE_URI" -a -n "$PGHOST" -a -n "$PGDATABASE" ]; then
  DATASOURCE_URI="jdbc:postgresql://$PGHOST"
  if [ -n "$PGPORT" ]; then
    DATASOURCE_URI+=":$PGPORT"
  fi

  DATASOURCE_URI+="/$PGDATABASE"
fi

if [ -n "$DATASOURCE_URI" ]; then
  JAVA_OPTS+=" -Dspring.datasource.url=$DATASOURCE_URI"
  if [ -n "$DATASOURCE_PARAMS" ]; then
   JAVA_OPTS+="?$DATASOURCE_PARAMS"
  fi
fi

if [ -n "$DATASOURCE_USER" -a -n "$DATASOURCE_PASS" ]; then
  JAVA_OPTS+=" -Dspring.datasource.username=$DATASOURCE_USER"
  JAVA_OPTS+=" -Dspring.datasource.password=$DATASOURCE_PASS"
fi

if [ -n "$FLYWAY_SET_BASELINE" ]; then
  JAVA_OPTS+=" -Dcbc.identity.dbMigration.setBaseline=$FLYWAY_SET_BASELINE"
fi

if [ -n "$FLYWAY_BASELINE_VERSION" ]; then
  JAVA_OPTS+=" -Dcbc.identity.dbMigration.baselineVersion=$FLYWAY_BASELINE_VERSION"
fi

if [ -n "$INIT_TEST_DATA" ]; then
  JAVA_OPTS+=" -Dcbc.identity.dbMigration.initTestData=$INIT_TEST_DATA"
fi

JAVA_OPTS+=" -XX:-OmitStackTraceInFastThrow"

if [ -n "$EXT_JAVA_OPTS" ]; then
  echo "External java options: $EXT_JAVA_OPTS"
  JAVA_OPTS+=" $EXT_JAVA_OPTS"
fi

echo "JAVA_OPTS: $JAVA_OPTS"

echo -e '\n\n==============\n\n\n'

cd "$IDENTITY_ROOT"
java \
    $JAVA_OPTS \
    -jar identity-db-migration.jar \
    "$@" &
PID=$!

EXITCODE=0
wait "$PID" || EXITCODE=$?
echo "Java process exited with $EXITCODE"
exit "$EXITCODE"
